﻿using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Routing;
using System;
using System.Globalization;
using System.Text.RegularExpressions;

namespace TestProject
{
    /// <summary>
    /// 커스텀 명칭 제약 조건
    /// </summary>
    public class CustomNameConstraint : IRouteConstraint
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 정규식
        /// </summary>
        private Regex regex;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - CustomNameConstraint()

        /// <summary>
        /// 생성자
        /// </summary>
        public CustomNameConstraint()
        {
            this.regex = new Regex
            (
                @"^[1-9]*$",
                RegexOptions.CultureInvariant | RegexOptions.IgnoreCase,
                TimeSpan.FromMilliseconds(100)
            );
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 매칭 여부 구하기 - Match(httpContext, route, routeKey, valueDictionary, routeDirection)

        /// <summary>
        /// 매칭 여부 구하기
        /// </summary>
        /// <param name="httpContext">HTTP 컨텍스트</param>
        /// <param name="router">라우터</param>
        /// <param name="routeKey">경로 키</param>
        /// <param name="valueDictionary">경로 값 딕셔너리</param>
        /// <param name="routeDirection">경로 방향</param>
        /// <returns>매칭 여부</returns>
        public bool Match
        (
            HttpContext          httpContext,
            IRouter              router,
            string               routeKey,
            RouteValueDictionary valueDictionary,
            RouteDirection       routeDirection
        )
        {
            if(valueDictionary.TryGetValue(routeKey, out object value))
            {
                string parameterValue = Convert.ToString(value, CultureInfo.InvariantCulture);

                if(parameterValue == null)
                {
                    return false;
                }

                return regex.IsMatch(parameterValue);
            }

            return false;
        }

        #endregion
    }
}